package afd.namesandnumbers;

import afd.nn.Engine;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Build;
import android.util.Log;
import android.widget.Toast;

public class SearchAsyncTask extends AsyncTask<Object, String, Void> {

	SearchForm activity;
	String fastfind;
	ProgressDialog progressDialog;
	ResultsCollection results;
	Engine engine;
	int retVal;			// value returned from Engine methods ... negative values => error occurred.
	Boolean reachedMaxResultLimit;	   // flag indicating that result-set is purposefully truncated.
	final Integer maxResultCount;	   // to be read in from a resource file.

	private String organisationSearch;
	private String surnameSearch;
	private String propertySearch;
	private String streetSearch;
	private String localitySearch;
	private String townSearch;
	private String countySearch;
	private String postcodeFromSearch;
	private String postcodeToSearch;

	public SearchAsyncTask(Activity searchActivity) {
		activity = (SearchForm) searchActivity;
		maxResultCount = activity.getResources().getInteger(R.integer.search_results_limit);
	}

	@Override
	protected void onPreExecute() {
		progressDialog = new ProgressDialog(activity);
		progressDialog.setMessage("Doing Search...");
		progressDialog.setCancelable(true);
		progressDialog.setIndeterminate(true);
		progressDialog.show();

		// extract strings from activity object
		// NOTE: cannot do this in "doInBackground" function because .getText runs in the UI-thread.
		organisationSearch = activity.organisationText.getText().toString();
		surnameSearch = activity.surnameText.getText().toString();
		propertySearch = activity.propertyText.getText().toString();
		streetSearch = activity.streetText.getText().toString();
		localitySearch = activity.localityText.getText().toString();
		townSearch = activity.townText.getText().toString();
		countySearch = activity.countyText.getText().toString();
		postcodeFromSearch = activity.postcodeFromText.getText().toString();
		postcodeToSearch = activity.postcodeToText.getText().toString();

	}

	@Override
	protected Void doInBackground(Object... params) {

		// Setup afd.nn.Engine class;
		engine = new afd.nn.Engine(Common.dataPath(activity.getApplicationContext()));
		engine.clear();
		engine.setOrganisation(organisationSearch);
		engine.setSurname(surnameSearch);
		engine.setProperty(propertySearch);
		engine.setStreet(streetSearch);
		engine.setLocality(localitySearch);
		engine.setTown(townSearch);
		engine.setCountyPostal(countySearch);
		engine.setPostcodeFrom(postcodeFromSearch);
		engine.setPostcode(postcodeToSearch);

		// Do the lookup.
		retVal = engine.addressGetFirst(0);

		// Deal with any error.
		if (retVal < 0) {
			cancel(true);
			return null;
		}

		reachedMaxResultLimit = false;
		Integer resultCount = 0;

		// Store results.
		results = new ResultsCollection();

		while (retVal >= 0) {
			if (retVal != Engine.AFD_RECORD_BREAK) {
				String addressList = engine.getAddressList();
				Log.i("AFD", "addressList = " + addressList);

				// Remove postcode and spaces from start of AddressList.
				String listItem = addressList.substring(13);
				String recordKey = engine.getRecordKey();

				ResultItem item = new ResultItem();
				item.listValue = listItem;
				item.recordKey = recordKey;

				results.add(item);

				resultCount++;
				if (resultCount > maxResultCount) {
					reachedMaxResultLimit = true;
					break;
				}
			}

			retVal = engine.addressGetNext();
		}
		return null;
	}

	@Override
	protected void onCancelled() {
		if (retVal < 0) {
			if (progressDialog != null && progressDialog.isShowing()) {
				progressDialog.dismiss();
			}

			AlertDialog.Builder dlgAlertBuilder = new AlertDialog.Builder(activity);
			dlgAlertBuilder.setTitle("Error");

			dlgAlertBuilder
					.setMessage(Engine.errorText(retVal))
					.setCancelable(true)
					.setNegativeButton("OK", new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int id) {
							dialog.cancel();
						}
					});
			AlertDialog dlgAlert = dlgAlertBuilder.create();
			dlgAlert.show();
		}
	}

	@Override
	protected void onPostExecute(Void result) {
		if (isCancelled() && Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB) {	// onCancelled is only supported since API level 11 (HoneyComb)
			onCancelled();																// so have to invoke it manually for earlier versions.
		} else {

			if (progressDialog != null && progressDialog.isShowing()) {
				progressDialog.dismiss();
			}




			int size;

			try {
				size = results.size();
			} catch (Exception e) {
				size = 0;
			}

			switch (size) {
				case 0:
					AlertDialog.Builder dlgAlert = new AlertDialog.Builder(activity);
					dlgAlert.setTitle("Results");
					dlgAlert.setMessage("No results found.");
					dlgAlert.setCancelable(false);
					dlgAlert.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.dismiss();
						}
					});
					dlgAlert.create().show();
					break;

				case 1:
					// Go directly to details.
					ResultItem item = results.getList().get(0);
					String recordKey = item.getRecordKey();

					Intent detailsIntent = new Intent(activity, DetailsView.class);
					detailsIntent.putExtra("RecordKey", recordKey);
					activity.startActivity(detailsIntent);
					break;

				default:
					Log.i("AFD", "Default");
					Intent resultsIntent = new Intent(activity, ResultsView.class);
					resultsIntent.putExtra("afd.namesandnumbers.ResultsCollection", results);
					activity.startActivity(resultsIntent);

					if (reachedMaxResultLimit) {
						Toast.makeText(activity.getApplicationContext(),
							"Only the first " + maxResultCount + " matches are displayed.  You may need to refine the search criteria.",
							Toast.LENGTH_LONG).show();
					}
					break;
			}
		}
	}
}
